==> sdlskk-0.1.tar.gz: sdlskk-0.1/romkana_table.c <==
#include "sdlskk_internal.h"
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

SDLSKK_RomKanaRuleTable* SDLSKK_RomKanaRuleTable_new( void )
{
  SDLSKK_RomKanaRuleTable* table;
  table = malloc( sizeof(SDLSKK_RomKanaRuleTable) );
  table->top = NULL;
  return table;
}

void SDLSKK_RomKanaRuleTable_delete( SDLSKK_RomKanaRuleTable* rule_table )
{
  SDLSKK_RomKanaElement* element;
  SDLSKK_RomKanaElement* next_element;
  
  for( element = rule_table->top; element != NULL;  element = next_element ){
    free( element->key );
    free( element->next_state );
    free( element->hiragana );
    free( element->katakana );
    next_element = element->next;
    free( element );
  }
}

void SDLSKK_RomKanaRuleTable_add( SDLSKK_RomKanaRuleTable* rule_table,
				  SDLSKK_Char* key,
				  SDLSKK_Char* next_state, 
				  SDLSKK_Char* hiragana,
				  SDLSKK_Char* katakana
				  )
{
  SDLSKK_RomKanaElement* new_element;

  new_element = malloc( sizeof(SDLSKK_RomKanaElement) );
  new_element->key = key;
  new_element->hiragana = hiragana;
  new_element->katakana = katakana;
  new_element->next_state = next_state;
  new_element->next = rule_table->top;
  rule_table->top = new_element;
}

static SDLSKK_Char* tokenize_tab_separated_line( char* line, int* pos )
{
  return SDLSKK_cstr_tokenize( line, '\t' ,pos );
}
  
void SDLSKK_RomKanaRuleTable_load( SDLSKK_RomKanaRuleTable* rule_table,
				   char* filename )
{
  FILE* fp;
  char buf[2048];
  SDLSKK_Char* prefix;
  SDLSKK_Char* next_state;
  SDLSKK_Char* hiragana;
  SDLSKK_Char* katakana;
  
  int pos;
  
  fp = fopen( filename, "rt" );
  if( fp==NULL )
    return;
  
  while( fgets( buf, sizeof(buf), fp ) != NULL ){
    if( buf[0]==';' || buf[0]==' ' || buf[0]=='\n' || buf[0]=='\0' )
      continue;
    pos = 0;
    prefix = tokenize_tab_separated_line( buf, &pos );
    next_state = tokenize_tab_separated_line( buf, &pos );
    katakana = tokenize_tab_separated_line( buf, &pos );
    hiragana = tokenize_tab_separated_line( buf, &pos );
    SDLSKK_RomKanaRuleTable_add( rule_table, prefix, next_state,
				 hiragana, katakana );
  }

  fclose(fp);
  return;
}

static SDLSKK_RomKanaSolveState RomKanaPrefixCheck( SDLSKK_Char* prefix,
						    SDLSKK_Char* key )
{
  int i;
  for( i=0; i< SDLSKK_Str_len( prefix ); ++i ){
    if( prefix[i] != key[i] )
      return SDLSKK_RomKana_NoCandidate;
  }
  if( key[i] == 0 )
    return SDLSKK_RomKana_OneCandidate;
  else
    return SDLSKK_RomKana_SomeCandidates;
}

SDLSKK_RomKanaSolveState SDLSKK_RomKanaRuleTable_get( SDLSKK_RomKanaRuleTable* rule_table,
						      SDLSKK_Char* prefix,
						      SDLSKK_RomKanaElement** candidate )
{
  SDLSKK_RomKanaSolveState solve_state;
  SDLSKK_RomKanaElement* element;

  solve_state = SDLSKK_RomKana_NoCandidate;
  *candidate = NULL;
  
  for( element = rule_table->top; element != NULL; element = element->next ){
    switch( RomKanaPrefixCheck( prefix, element->key ) ){
    case SDLSKK_RomKana_NoCandidate:
      break;
    case SDLSKK_RomKana_OneCandidate:
      switch( solve_state ){
      case SDLSKK_RomKana_NoCandidate:
	solve_state = SDLSKK_RomKana_OneCandidate;
	*candidate = element;
	break;
      case SDLSKK_RomKana_SomeCandidates:
	solve_state = SDLSKK_RomKana_UnresolvedCandidate;
	*candidate = element;
	break;
      case SDLSKK_RomKana_OneCandidate:
	/* RomKanaTable is not regular */
	assert(0);
	break;
      case SDLSKK_RomKana_UnresolvedCandidate:
	/* RomKanaTable is not regular */
	assert(0);
	break;
      }
      break;
    case SDLSKK_RomKana_SomeCandidates:
      switch( solve_state ){
      case SDLSKK_RomKana_NoCandidate:
	solve_state = SDLSKK_RomKana_SomeCandidates;
	break;
      case SDLSKK_RomKana_SomeCandidates:
	break;
      case SDLSKK_RomKana_OneCandidate:
	solve_state = SDLSKK_RomKana_UnresolvedCandidate;
	break;
      case SDLSKK_RomKana_UnresolvedCandidate:
	break;
      }
      break;
    case SDLSKK_RomKana_UnresolvedCandidate:
      assert(0);
      break;
    }
  }
  return solve_state;
}

==> sdlskk-0.2.tar.gz: sdlskk-0.2/romkana_table.c <==
#include "sdlskk_internal.h"
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

void SDLSKK_RomKanaRuleTable_delete( SDLSKK_RomKanaRuleTable* rule_table )
{
  SDLSKK_RomKanaElement* element;
  SDLSKK_RomKanaElement* next_element;
  
  for( element = rule_table->top; element != NULL;  element = next_element ){
    free( element->key );
    free( element->next_state );
    free( element->hiragana );
    free( element->katakana );
    next_element = element->next;
    free( element );
  }
}

void SDLSKK_RomKanaRuleTable_add( SDLSKK_RomKanaRuleTable* rule_table,
				  SDLSKK_Char* key,
				  SDLSKK_Char* next_state, 
				  SDLSKK_Char* hiragana,
				  SDLSKK_Char* katakana
				  )
{
  SDLSKK_RomKanaElement* new_element;

  new_element = SDLSKK_malloc( sizeof(SDLSKK_RomKanaElement) );
  new_element->key = key;
  new_element->hiragana = hiragana;
  new_element->katakana = katakana;
  new_element->next_state = next_state;
  new_element->next = rule_table->top;
  rule_table->top = new_element;
}

static SDLSKK_Char* tokenize_tab_separated_line( char* line, int* pos )
{
  return SDLSKK_cstr_tokenize( line, '\t' ,pos );
}
  
static int  SDLSKK_RomKanaRuleTable_load( SDLSKK_RomKanaRuleTable* rule_table,
					  char* filename )
{
  FILE* fp;
  char buf[2048];
  SDLSKK_Char* prefix;
  SDLSKK_Char* next_state;
  SDLSKK_Char* hiragana;
  SDLSKK_Char* katakana;
  
  int pos;
  
  fp = fopen( filename, "rt" );
  if( fp==NULL )
    return 0;
  
  while( fgets( buf, sizeof(buf), fp ) != NULL ){
    if( ( buf[0]==';' && buf[1]==';' ) || buf[0]==' ' ||
	buf[0]=='\n' || buf[0]=='\0' )
      continue;
    pos = 0;
    prefix = tokenize_tab_separated_line( buf, &pos );
    next_state = tokenize_tab_separated_line( buf, &pos );
    katakana = tokenize_tab_separated_line( buf, &pos );
    hiragana = tokenize_tab_separated_line( buf, &pos );
    SDLSKK_RomKanaRuleTable_add( rule_table, prefix, next_state,
				 hiragana, katakana );
  }

  fclose(fp);
  return 1;
}

SDLSKK_RomKanaRuleTable* SDLSKK_RomKanaRuleTable_new( char* filename )
{
  SDLSKK_RomKanaRuleTable* table;
  table = SDLSKK_malloc( sizeof(SDLSKK_RomKanaRuleTable) );
  table->top = NULL;
  if( SDLSKK_RomKanaRuleTable_load( table, filename ) ){
    return table;
  }else{
    SDLSKK_RomKanaRuleTable_delete( table );
    return NULL;
  }
}

static SDLSKK_RomKanaSolveState RomKanaPrefixCheck( SDLSKK_Char* prefix,
						    SDLSKK_Char* key )
{
  int i;
  for( i=0; i< SDLSKK_Str_len( prefix ); ++i ){
    if( prefix[i] != key[i] )
      return SDLSKK_RomKana_NoCandidate;
  }
  if( key[i] == 0 )
    return SDLSKK_RomKana_OneCandidate;
  else
    return SDLSKK_RomKana_SomeCandidates;
}

SDLSKK_RomKanaSolveState SDLSKK_RomKanaRuleTable_get( SDLSKK_RomKanaRuleTable* rule_table,
						      SDLSKK_Char* prefix,
						      SDLSKK_RomKanaElement** candidate )
{
  SDLSKK_RomKanaSolveState solve_state;
  SDLSKK_RomKanaElement* element;

  solve_state = SDLSKK_RomKana_NoCandidate;
  *candidate = NULL;
  
  for( element = rule_table->top; element != NULL; element = element->next ){
    switch( RomKanaPrefixCheck( prefix, element->key ) ){
    case SDLSKK_RomKana_NoCandidate:
      break;
    case SDLSKK_RomKana_OneCandidate:
      switch( solve_state ){
      case SDLSKK_RomKana_NoCandidate:
	solve_state = SDLSKK_RomKana_OneCandidate;
	*candidate = element;
	break;
      case SDLSKK_RomKana_SomeCandidates:
	solve_state = SDLSKK_RomKana_UnresolvedCandidate;
	*candidate = element;
	break;
      case SDLSKK_RomKana_OneCandidate:
	/* RomKanaTable is not regular */
	assert(0);
	break;
      case SDLSKK_RomKana_UnresolvedCandidate:
	/* RomKanaTable is not regular */
	assert(0);
	break;
      }
      break;
    case SDLSKK_RomKana_SomeCandidates:
      switch( solve_state ){
      case SDLSKK_RomKana_NoCandidate:
	solve_state = SDLSKK_RomKana_SomeCandidates;
	break;
      case SDLSKK_RomKana_SomeCandidates:
	break;
      case SDLSKK_RomKana_OneCandidate:
	solve_state = SDLSKK_RomKana_UnresolvedCandidate;
	break;
      case SDLSKK_RomKana_UnresolvedCandidate:
	break;
      }
      break;
    case SDLSKK_RomKana_UnresolvedCandidate:
      assert(0);
      break;
    }
  }
  return solve_state;
}
